#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['ebmdatalab', 'ebmdatalab.tests']

package_data = \
{'': ['*'], 'ebmdatalab': ['data/*']}

install_requires = \
['statsmodels',
 'seaborn',
 'pandas >= 0.24',
 'pandas-gbq >= 0.8.0',
 'geopandas',
 'descartes']

extras_require = \
{'test': ['pytest >=2.7.3']}

setup(name='ebmdatalab',
      version='0.0.20',
      description='Package for ebmdatalab jupyter notebook stuff',
      author='EBM DataLab',
      author_email='tech@ebmdatalab.net',
      url='https://github.com/ebmdatalab/datalab-pandas',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
     )
